/*
 * Copyright 2011 JOHK Software
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package com.wesleyhome.johksoftware.utilities;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInput;
import java.io.ObjectInputStream;
import java.io.ObjectOutput;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import net.iharder.Base64;
import org.apache.commons.io.FileUtils;

/**
 * @author jwesley
 *
 */
public final class ObjectSerializer<E extends Serializable> {

	private Class<E> objectClass;
	private File objectFile;

	public static <T extends Serializable> ObjectSerializer<T> serialize(final T object, final File objectFile) throws IOException {
		Class<T> cls = (Class<T>) object.getClass();
		ObjectSerializer<T> serializer = new ObjectSerializer<T>(cls, objectFile);
		serializer.serializeObject(object);
		return serializer;
	}

	/**
	 * 
	 */
	public ObjectSerializer(final Class<E> objectClass, final File objectFile) {
		this.objectClass = objectClass;
		this.objectFile = objectFile;
	}

	public void serializeObject(final E object) throws IOException {
		File parentFile = objectFile.getParentFile();
		if (parentFile != null) {
			FileUtils.forceMkdir(parentFile);
		}
		if (!objectFile.exists()) {
			objectFile.createNewFile();
		}
		String encodeObject = Base64.encodeObject(object);
		ObjectOutput objectOutput = new ObjectOutputStream(new FileOutputStream(objectFile));
		objectOutput.writeObject(encodeObject);
		objectOutput.flush();
		objectOutput.close();
	}

	@SuppressWarnings("unchecked")
	public E unserializeObject() {
		try {
			if (objectFile.exists()) {
				ObjectInput objectInput = new ObjectInputStream(new FileInputStream(objectFile));
				String readObject = (String) objectInput.readObject();
				E decodeToObject = (E) Base64.decodeToObject(readObject);
				return decodeToObject;
			}
			return objectClass.newInstance();
		} catch (InstantiationException e) {
			return null;
		} catch (IllegalAccessException e) {
			return null;
		} catch (FileNotFoundException e) {
			return null;
		} catch (IOException e) {
			return null;
		} catch (ClassNotFoundException e) {
			return null;
		}
	}
}
